// Copyright 2019 The Flutter team. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

import 'dart:math' as math;

import 'package:align_positioned/align_positioned.dart';
import 'package:flutter/material.dart';
import 'package:pc_plugins/utils/utils.dart';


/// A colored piece of the [RallyPieChart].
class RallyPieChartSegment {
  const RallyPieChartSegment({required this.color,
    required this.value,required this.title});

  final String title;
  final Color color;
  final double value;
}

/// The max height and width of the [RallyPieChart].
const pieChartMaxSize = 500.0;


/// An animated circular pie chart to represent pieces of a whole, which can
/// have empty space.
///
/// heroLabel
/// heroAmount

class RallyPieChart extends StatefulWidget {
  const RallyPieChart(
      {required this.heroLabel,required this.heroAmount,
        required this.wholeAmount,required this.segments,
        this.showLabel=true});

  final String heroLabel;
  final String heroAmount;
  final double wholeAmount;
  final List<RallyPieChartSegment> segments;
  final bool showLabel;

  @override
  _RallyPieChartState createState() => _RallyPieChartState();
}

class _RallyPieChartState extends State<RallyPieChart>
    with SingleTickerProviderStateMixin {
  late AnimationController controller;
  late Animation<double> animation;

  @override
  void initState() {
    super.initState();
    controller = AnimationController(
      duration: const Duration(milliseconds: 626),
      vsync: this,
    );
    animation = CurvedAnimation(
        parent: TweenSequence<double>(<TweenSequenceItem<double>>[
          TweenSequenceItem<double>(
            tween: Tween<double>(begin: 0, end: 0),
            weight: 1,
          ),
          TweenSequenceItem<double>(
            tween: Tween<double>(begin: 0, end: 1),
            weight: 1.5,
          ),
        ]).animate(controller),
        curve: Curves.decelerate);
    controller.forward();
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return MergeSemantics(
      child: _AnimatedRallyPieChart(
        showLabel:widget.showLabel,
        animation: animation,
        centerLabel: widget.heroLabel,
        centerAmount: widget.heroAmount,
        total: widget.wholeAmount,
        segments: widget.segments,
      ),
    );
  }
}

class _AnimatedRallyPieChart extends AnimatedWidget {
  const _AnimatedRallyPieChart({
    Key? key,
    required this.animation,
    required this.centerLabel,
    required this.centerAmount,
    required this.total,
    required this.segments,
    this.showLabel=true
  }) : super(key: key, listenable: animation);

//  static const double margn=1;
  final bool showLabel;
  final Animation<double> animation;
  final String centerLabel;
  final String centerAmount;
  final double total;
  final List<RallyPieChartSegment> segments;

  static const showTextThres=0;
  static const double dOffset=8;

  @override
  Widget build(BuildContext context) {
    final textTheme = Theme.of(context).textTheme;
    final labelTextStyle = textTheme.bodyText2?.copyWith(
      fontSize: 14,);

    return LayoutBuilder(builder: (context, constraints) {
      // When the widget is larger, we increase the font size.
      var headlineStyle = constraints.maxHeight >= pieChartMaxSize
          ? textTheme.headline5?.copyWith(fontSize: 70)
          : textTheme.headline5;


      final double side=math.min(constraints.maxWidth, constraints.maxWidth);
      double sum=0;
      double offset=dOffset;
      double last=0;
      List<Widget> labels=[];

      if(showLabel)
      for(int i=0;i<segments.length;i++){
        if((segments[i].value+last)/2.0/total<0.1) offset+=dOffset;
        else offset=0;
        if(segments[i].value>showTextThres)
          labels.add(
              label(context,sum+segments[i].value/2.0,offset, segments[i], constraints.maxHeight, animation.value));
        sum+=segments[i].value;
        last=segments[i].value;
      }

      return DecoratedBox(
        decoration: _RallyPieChartOutlineDecoration(
          bgColor: cTheme.of(context).data.scf_bg,
          maxFraction: animation.value,
          total: total,
          segments: segments,
        ),
        child: Container(
          height: constraints.maxHeight,
          width: constraints.maxHeight,
          alignment: Alignment.center,
          child: Stack(children:[
            if(showLabel)...labels,
        AlignPositioned(
            alignment: Alignment.center,
            child:Column(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              Text(
                centerLabel,
                style: labelTextStyle,
              ),
              Text(
                centerAmount,
//                usdWithSignFormat(context).format(centerAmount),
                style: headlineStyle,
              ),
            ],
          ))]),
        ),
      );
    });
  }

  static const shortBarOs=0.027;
  Widget label(BuildContext context,double sm,double offset,
      RallyPieChartSegment item,double height,double prg){

    String text='${item.title}: ${(item.value/total*100).floor()}%';

    final dgr=sm/total;

    bool addHrz=!(dgr>0.25-shortBarOs&&dgr<0.25+shortBarOs)&&!(dgr>0.75-shortBarOs&&dgr<0.75+shortBarOs);
    ///for text at 90,270 degrees
    double hori_dilta=(1-math.cos(dgr*math.pi*4))*text.length*0.9;
    ///for text at 0,360 degrees
    double vert_dilta=(math.cos(dgr*math.pi*4)+1)*offset+(isDesktop(context)?12:4.27);
    return AlignPositioned(
        alignment: Alignment.center,
        dy: -(height*prg/2 )+_RallyPieChartOutlineBoxPainter.strokeWidth*3,
//        dx:math.cos(sm/total*math.pi*2)*27,
        rotateDegrees: dgr*360,
        moveByChildHeight: -0.5,
        child://Container(width:26,height:27,color:Colors.white)

        Column(mainAxisSize:MainAxisSize.min,children: [Container(
          height: 27,width: 160,
            child:AlignPositioned(
            alignment: Alignment.center,
            rotateDegrees: -dgr*360,
            child:Text(text,maxLines: 2,
          style: Theme.of(context).textTheme.caption?.copyWith(fontSize: 9),))),
          Container(width: 2.7,height: hori_dilta,
            color: prg==1&&addHrz?textColor.withOpacity(0.6):Colors.transparent,),
          Container(width: 2.7,height: vert_dilta,
            color: prg==1?textColor.withOpacity(0.6):Colors.transparent,),
        ])
    );
  }
}

class _RallyPieChartOutlineDecoration extends Decoration {
  const _RallyPieChartOutlineDecoration(
      {required this.maxFraction,required this.total,
        required this.segments,required this.bgColor});

  final Color bgColor;
  final double maxFraction;
  final double total;
  final List<RallyPieChartSegment> segments;


  @override
  BoxPainter createBoxPainter([VoidCallback? onChanged]) {
    return _RallyPieChartOutlineBoxPainter(
      bgColor: bgColor,
      maxFraction: maxFraction,
      wholeAmount: total,
      segments: segments,
    );
  }
}

class _RallyPieChartOutlineBoxPainter extends BoxPainter {
  _RallyPieChartOutlineBoxPainter(
      {required this.maxFraction,required this.wholeAmount,
        required this.segments,required this.bgColor});

  final Color bgColor;
  final double maxFraction;
  final double wholeAmount;
  final List<RallyPieChartSegment> segments;
  static const double wholeRadians = 2 * math.pi;
  static const double spaceRadians = wholeRadians / 180;

  static const strokeWidth = 4.0;
  @override
  void paint(Canvas canvas, Offset offset, ImageConfiguration configuration) {
    // Create two padded reacts to draw arcs in: one for colored arcs and one for
    // inner bg arc.

    final outerRadius = math.min(
          configuration.size!.width,
          configuration.size!.height,
        ) /
        2;
    final outerRect = Rect.fromCircle(
      center: configuration.size!.center(offset),
      radius: outerRadius - strokeWidth * 3,
    );
    final innerRect = Rect.fromCircle(
      center: configuration.size!.center(offset),
      radius: outerRadius - strokeWidth * 4,
    );

    // Paint each arc with spacing.
    var cumulativeSpace = 0.0;
    var cumulativeTotal = 0.0;
    for (final segment in segments) {
      final paint = Paint()..color = segment.color;
      final startAngle = _calculateStartAngle(cumulativeTotal, cumulativeSpace);
      final sweepAngle = _calculateSweepAngle(segment.value, 0);
      canvas.drawArc(outerRect, startAngle, sweepAngle, true, paint);
      cumulativeTotal += segment.value;
      cumulativeSpace += spaceRadians;
    }

    // Paint any remaining space black (e.g. budget amount remaining).
    final remaining = wholeAmount - cumulativeTotal;
    if (remaining > 0) {
      final paint = Paint()..color = Colors.black;
      final startAngle =
          _calculateStartAngle(cumulativeTotal, spaceRadians * segments.length);
      final sweepAngle = _calculateSweepAngle(remaining, -spaceRadians);
      canvas.drawArc(outerRect, startAngle, sweepAngle, true, paint);
    }

    // Paint a smaller inner circle to cover the painted arcs, so they are
    // display as segments.
    final bgPaint = Paint()..color = bgColor;
    canvas.drawArc(innerRect, 0, 2 * math.pi, true, bgPaint);
  }

  double _calculateAngle(double amount, double offset) {
    final wholeMinusSpacesRadians =
        wholeRadians - (segments.length * spaceRadians);
    return maxFraction *
        (amount / wholeAmount * wholeMinusSpacesRadians + offset);
  }

  double _calculateStartAngle(double total, double offset) =>
      _calculateAngle(total, offset) - math.pi / 2;

  double _calculateSweepAngle(double total, double offset) =>
      _calculateAngle(total, offset);
}
